/*
 * Copyright (c) 2011 Michael Pippig
 *
 * This file is part of PNFFT.
 *
 * PNFFT is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * PNFFT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PNFFT.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "pnfft.h"
#include "ipnfft.h"

static void fft_output_size(
    const INT *n, const R *x_max, int m,
    INT *no);


void PNX(local_size_guru)(
    int d, const INT *N, const INT *n, const R *x_max, int m,
    MPI_Comm comm_cart,
    INT *local_N, INT *local_N_start,
    R *lower_border, R *upper_border
    )
{
  INT no[3], local_no[3], local_no_start[3];

  if(d != 3){
    PX(fprintf)(comm_cart, stderr, "!!! Error in PNFFT: d != 3 not yet implemented !!!\n");
    return;
  }

  fft_output_size(n, x_max, m,
      no);

  PNX(local_size_internal)(N, n, no, comm_cart,
      local_N, local_N_start, local_no, local_no_start);

  PNX(node_borders)(n, no, local_no, local_no_start,
      lower_border, upper_border);
}


void PNX(init_guru)(
    PNX(plan) *ths, int d, const INT *N, const INT *n, const R *x_max,
    INT local_M, int m,
    unsigned nfft_flags, unsigned fftw_flags,
    MPI_Comm comm_cart
    )
{
  INT no[3];
  
  if(d != 3){
    PX(fprintf)(comm_cart, stderr, "!!! Error in PNFFT: d != 3 not yet implemented !!!\n");
    return;
  }
  
  fft_output_size(n, x_max, m,
    no);

  PNX(init_internal)(ths, d, N, n, no, local_M, m, nfft_flags, fftw_flags, comm_cart);
}


static void fft_output_size(
    const INT *n, const R *x_max, int m,
    INT *no
    )
{
  INT c;

  for(int t=0; t<3; t++){
    c = lrint(floor(n[t]*x_max[t]));
    no[t] = MIN(n[t], 2*(c+m+2));
  }
}


