#
# Copyright (c) 2008 Jens Keiner
# Copyright (c) 2010 Michael Pippig
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#  If we find the library, set the shell variable `ax_have_pfft' to `yes'.
#  Otherwise, set `ax_have_pfft' to `no'.  If the argument to --with-pfft
#  is `built', then assume pfft_include_dir and pfft_lib_dir have been set
#  for a tree in which pfft is built alongside.

AC_DEFUN([AX_LIB_PFFT],
[
  AC_REQUIRE([AX_LIB_FFTW3]) 
 
  AC_ARG_WITH(pfft, [AC_HELP_STRING([--with-pfft=DIR],
    [compile with pfft in DIR])], with_pfft=$withval, with_pfft="yes")

  AC_ARG_WITH(pfft-libdir, [AC_HELP_STRING([--with-pfft-libdir=DIR],
    [compile with pfft library directory DIR])], pfft_lib_dir=$withval,
    pfft_lib_dir="yes")

  AC_ARG_WITH(pfft-includedir, [AC_HELP_STRING([--with-pfft-includedir=DIR],
    [compile with pfft include directory DIR])], pfft_include_dir=$withval,
    pfft_include_dir="yes")

  if test "x$with_pfft" = xbuilt; then
    if test -z "$pfft_include_dir" || test -z "$pfft_lib_dir"; then
      AC_MSG_ERROR([for an in-tree pfft, set pfft_include_dir and pfft_lib_dir])
    fi
    pfft_CPPFLAGS="-I$pfft_include_dir"
    pfft_LDFLAGS="-L$pfft_lib_dir"
    pfft_LIBS="-lpfft"
  else
    if test "x$with_pfft" != "xyes"; then
      if test "x${pfft_include_dir}" = "xyes"; then
        pfft_include_dir="$with_pfft/include"
      fi
      if test "x${pfft_lib_dir}" = "xyes"; then 
        pfft_lib_dir="$with_pfft/lib"
      fi
    fi
  
    if test "x${pfft_include_dir}" != "xyes"; then
      AX_CHECK_DIR([${pfft_include_dir}],[],
        [AC_MSG_ERROR([The directory ${pfft_include_dir} does not exist.])])
      pfft_CPPFLAGS="-I$pfft_include_dir"
    else
      pfft_CPPFLAGS=""
    fi
  
    if test "x${pfft_lib_dir}" != "xyes"; then 
      AX_CHECK_DIR([${pfft_lib_dir}],[],
        [AC_MSG_ERROR([The directory ${pfft_lib_dir} does not exist.])])
      pfft_LDFLAGS="-L$pfft_lib_dir"
    else
      pfft_LDFLAGS=""
    fi
  
    saved_LDFLAGS="$LDFLAGS"
    saved_CPPFLAGS="$CPPFLAGS"
    saved_LIBS="$LIBS"
    CPPFLAGS="$CPPFLAGS $fftw3_CPPFLAGS $pfft_CPPFLAGS"
    LDFLAGS="$LDFLAGS $fftw3_LDFLAGS $pfft_LDFLAGS"
    LIBS="$fftw3_mpi_LIBS $fftw3_LIBS $LIBS"
  
    # Check if header is present and usable.
    ax_lib_pfft=yes
    AC_CHECK_HEADER([pfft.h], [], [ax_lib_pfft=no;])
  
    if test "x$ax_lib_pfft" = "xyes"; then
      AC_CHECK_LIB([pfft], [pfft_execute], [], [ax_lib_pfft=no])
      pfft_LIBS="-lpfft"
    fi
  
    # Restore saved flags.
    CPPFLAGS="$saved_CPPFLAGS"
    LDFLAGS="$saved_LDFLAGS"
    LIBS="$saved_LIBS"
  fi

  AC_SUBST(pfft_LIBS)
  AC_SUBST(pfft_LDFLAGS)
])
